@@ -79,7 +79,6 @@ extern struct user_settings *user_settings;
 /* Maximum allowable size of the nodes list */
 #define MAX_NODELIST_SIZE (MAX_RECV_CURL_DATA_SIZE)
 
-
 static struct Thread_Data {
     pthread_t tid;
     pthread_attr_t attr;
@@ -213,12 +212,47 @@ static int curl_fetch_nodes_JSON(struct Recv_Curl_Data *recv_data)
     headers = curl_slist_append(headers, "Content-Type: application/json");
     headers = curl_slist_append(headers, "charsets: utf-8");
 
-    curl_easy_setopt(c_handle, CURLOPT_HTTPHEADER, headers);
-    curl_easy_setopt(c_handle, CURLOPT_URL, NODES_LIST_URL);
-    curl_easy_setopt(c_handle, CURLOPT_WRITEFUNCTION, curl_cb_write_data);
-    curl_easy_setopt(c_handle, CURLOPT_WRITEDATA, recv_data);
-    curl_easy_setopt(c_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");
-    curl_easy_setopt(c_handle, CURLOPT_HTTPGET, 1L);
+    int ret = curl_easy_setopt(c_handle, CURLOPT_HTTPHEADER, headers);
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set http headers (libcurl error %d)", ret);
+        goto on_exit;
+    }
+
+    ret = curl_easy_setopt(c_handle, CURLOPT_URL, NODES_LIST_URL);
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set url (libcurl error %d)", ret);
+        goto on_exit;
+    }
+
+    ret = curl_easy_setopt(c_handle, CURLOPT_WRITEFUNCTION, curl_cb_write_data);
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set write function callback (libcurl error %d)", ret);
+        goto on_exit;
+    }
+
+    ret = curl_easy_setopt(c_handle, CURLOPT_WRITEDATA, recv_data);
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set write data (libcurl error %d)", ret);
+        goto on_exit;
+    }
+
+    ret = curl_easy_setopt(c_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set useragent (libcurl error %d)", ret);
+        goto on_exit;
+    }
+
+    ret = curl_easy_setopt(c_handle, CURLOPT_HTTPGET, 1L);
+
+    if (ret != CURLE_OK) {
+        fprintf(stderr, "Failed to set get request (libcurl error %d)", ret);
+        goto on_exit;
+    }
 
     int proxy_ret = set_curl_proxy(c_handle, arg_opts.proxy_address, arg_opts.proxy_port, arg_opts.proxy_type);
 
@@ -227,7 +261,7 @@ static int curl_fetch_nodes_JSON(struct Recv_Curl_Data *recv_data)
         goto on_exit;
     }
 
-    int ret = curl_easy_setopt(c_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2);
+    ret = curl_easy_setopt(c_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2);
 
     if (ret != CURLE_OK) {
         fprintf(stderr, "TLSv1.2 could not be set (libcurl error %d)", ret);
@@ -246,7 +280,13 @@ static int curl_fetch_nodes_JSON(struct Recv_Curl_Data *recv_data)
     if (ret != CURLE_OK) {
         /* If system doesn't support any of the specified ciphers suites, fall back to default */
         if (ret == CURLE_SSL_CIPHER) {
-            curl_easy_setopt(c_handle, CURLOPT_SSL_CIPHER_LIST, NULL);
+            ret = curl_easy_setopt(c_handle, CURLOPT_SSL_CIPHER_LIST, NULL);
+
+            if (ret != CURLE_OK) {
+                fprintf(stderr, "Failed to set SSL cipher list (libcurl error %d)\n", ret);
+                goto on_exit;
+            }
+
             ret = curl_easy_perform(c_handle);
         }
 
@@ -523,10 +563,14 @@ void *load_nodeslist_thread(void *data)
         pthread_mutex_unlock(&thread_data.lock);
     }
 
+    pthread_mutex_lock(&thread_data.lock);
+    const size_t num_nodes = Nodes.count;
+    pthread_mutex_unlock(&thread_data.lock);
+
     /* If nodeslist does not contain any valid entries we set the last_scan value
      * to 0 so that it will fetch a new list the next time this function is called.
      */
-    if (Nodes.count == 0) {
+    if (num_nodes == 0) {
         const char *s = "{\"last_scan\":0}";
         rewind(fp);
         fwrite(s, strlen(s), 1, fp);  // Not much we can do if it fails
@@ -593,12 +637,12 @@ static void DHT_bootstrap(Tox *m)
         return;
     }
 
-    size_t i;
-
     pthread_mutex_lock(&thread_data.lock);
 
-    for (i = 0; i < NUM_BOOTSTRAP_NODES; ++i) {
-        struct Node *node = &Nodes.list[rand() % Nodes.count];
+    for (size_t i = 0; i < NUM_BOOTSTRAP_NODES; ++i) {
+        const unsigned int idx = rand_range_not_secure(Nodes.count);
+        struct Node *node = &Nodes.list[idx];
+
         const char *addr = node->have_ip4 ? node->ip4 : node->ip6;
 
         if (!addr) {
