@@ -24,7 +24,6 @@
 #define _GNU_SOURCE    /* needed for wcswidth() */
 #endif
 
-#include <assert.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
@@ -67,13 +66,18 @@ static void kill_infobox(ToxWindow *self);
 static const char *chat_cmd_list[] = {
     "/accept",
     "/add",
+    "/autoaccept",
     "/avatar",
     "/cancel",
+    "/cinvite",
+    "/cjoin",
     "/clear",
     "/close",
     "/connect",
     "/exit",
+    "/gaccept",
     "/conference",
+    "/group",
 #ifdef GAMES
     "/game",
     "/play",
@@ -130,7 +134,7 @@ static void set_self_typingstatus(ToxWindow *self, Tox *m, bool is_typing)
 
     ChatContext *ctx = self->chatwin;
 
-    TOX_ERR_SET_TYPING err;
+    Tox_Err_Set_Typing err;
     tox_self_set_typing(m, self->num, is_typing, &err);
 
     if (err != TOX_ERR_SET_TYPING_OK) {
@@ -502,17 +506,12 @@ static void chat_onFileControl(ToxWindow *self, Tox *m, uint32_t friendnum, uint
     }
 
     switch (control) {
-        case TOX_FILE_CONTROL_RESUME: {
-            /* transfer is accepted */
+        case TOX_FILE_CONTROL_RESUME: {    /* transfer is accepted */
             if (ft->state == FILE_TRANSFER_PENDING) {
                 ft->state = FILE_TRANSFER_STARTED;
                 line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "File transfer [%zu] for '%s' accepted.",
                               ft->index, ft->file_name);
-                char progline[MAX_STR_SIZE];
-                init_progress_bar(progline);
-                line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s", progline);
                 sound_notify(self, silent, NT_NOFOCUS | user_settings->bell_on_filetrans_accept | NT_WNDALERT_2, NULL);
-                ft->line_id = self->chatwin->hst->line_end->id + 2;
             } else if (ft->state == FILE_TRANSFER_PAUSED) {    /* transfer is resumed */
                 ft->state = FILE_TRANSFER_STARTED;
             }
@@ -676,8 +675,10 @@ static void chat_onFileRecv(ToxWindow *self, Tox *m, uint32_t friendnum, uint32_
     FILE *filecheck = NULL;
     int count = 1;
 
-    while ((filecheck = fopen(file_path, "r"))) {
-        fclose(filecheck);
+    while ((filecheck = fopen(file_path, "r")) || file_transfer_recv_path_exists(file_path)) {
+        if (filecheck) {
+            fclose(filecheck);
+        }
 
         file_path[path_len] = '\0';
         char d[5];
@@ -700,8 +701,6 @@ static void chat_onFileRecv(ToxWindow *self, Tox *m, uint32_t friendnum, uint32_
         }
     }
 
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Type '/savefile %zu' to accept the file transfer.", ft->index);
-
     ft->file_size = file_size;
     snprintf(ft->file_path, sizeof(ft->file_path), "%s", file_path);
     snprintf(ft->file_name, sizeof(ft->file_name), "%s", filename);
@@ -716,6 +715,19 @@ static void chat_onFileRecv(ToxWindow *self, Tox *m, uint32_t friendnum, uint32_
         box_notify(self, transfer_pending, NT_WNDALERT_0 | NT_NOFOCUS | user_settings->bell_on_filetrans,
                    &self->active_box, self->name, "Incoming file: %s", filename);
     }
+
+    const bool auto_accept_files = friend_get_auto_accept_files(friendnum);
+
+    if (auto_accept_files) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Auto-accepting file transfer %zu", ft->index);
+
+        char cmd[MAX_STR_SIZE];
+        snprintf(cmd, sizeof(cmd), "/savefile %zu", ft->index);
+        execute(self->window, self, m, cmd, CHAT_COMMAND_MODE);
+    } else {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0,
+                      "Type '/savefile %zu' to accept the file transfer.", ft->index);
+    }
 }
 
 static void chat_onConferenceInvite(ToxWindow *self, Tox *m, int32_t friendnumber, uint8_t type,
@@ -755,8 +767,46 @@ static void chat_onConferenceInvite(ToxWindow *self, Tox *m, int32_t friendnumbe
                    "invites you to join %s", description);
     }
 
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s has invited you to %s.", name, description);
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/join\" to join the chat.");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s has invited you to a conference.", name);
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/cjoin\" to join the chat.");
+}
+
+static void chat_onGroupInvite(ToxWindow *self, Tox *m, uint32_t friendnumber, const char *invite_data, size_t length,
+                               const char *group_name, size_t group_name_length)
+{
+    UNUSED_VAR(group_name_length);
+
+    if (self->num != friendnumber) {
+        return;
+    }
+
+    if (Friends.list[friendnumber].group_invite.data) {
+        free(Friends.list[friendnumber].group_invite.data);
+    }
+
+    Friends.list[friendnumber].group_invite.data = malloc(length * sizeof(uint8_t));
+
+    if (Friends.list[friendnumber].group_invite.data == NULL) {
+        return;
+    }
+
+    memcpy(Friends.list[friendnumber].group_invite.data, invite_data, length);
+    Friends.list[friendnumber].group_invite.length = length;
+
+    sound_notify(self, generic_message, NT_WNDALERT_2 | user_settings->bell_on_invite, NULL);
+
+    char name[TOX_MAX_NAME_LENGTH];
+    get_nick_truncate(m, name, friendnumber);
+
+    if (self->active_box != -1) {
+        box_silent_notify2(self, NT_WNDALERT_2 | NT_NOFOCUS, self->active_box, "invites you to join group chat");
+    } else {
+        box_silent_notify(self, NT_WNDALERT_2 | NT_NOFOCUS, &self->active_box, name, "invites you to join group chat");
+    }
+
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s has invited you to join group chat \"%s\"", name, group_name);
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0,
+                  "Type \"/gaccept <password>\" to join the chat (password is optional).");
 }
 
 #ifdef GAMES
@@ -780,7 +830,7 @@ void chat_onGameInvite(ToxWindow *self, Tox *m, uint32_t friend_number, const ui
 
     GameType type = data[1];
 
-    if (!game_type_is_multiplayer(type)) {
+    if (!game_type_has_multiplayer(type)) {
         return;
     }
 
@@ -1383,7 +1433,7 @@ static void chat_onDraw(ToxWindow *self, Tox *m)
     self->x = x2;
 
     /* Truncate note if it doesn't fit in statusbar */
-    size_t maxlen = x2 - getcurx(statusbar->topline) - (KEY_IDENT_DIGITS * 2) - 6;
+    size_t maxlen = x2 - getcurx(statusbar->topline) - KEY_IDENT_BYTES - 6;
 
     pthread_mutex_lock(&Winthread.lock);
     size_t statusmsg_len = statusbar->statusmsg_len;
@@ -1414,10 +1464,10 @@ static void chat_onDraw(ToxWindow *self, Tox *m)
     int s_x;
     getyx(statusbar->topline, s_y, s_x);
 
-    mvwhline(statusbar->topline, s_y, s_x, ' ', x2 - s_x - (KEY_IDENT_DIGITS * 2) - 3);
+    mvwhline(statusbar->topline, s_y, s_x, ' ', x2 - s_x - KEY_IDENT_BYTES  - 3);
     wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
 
-    wmove(statusbar->topline, 0, x2 - (KEY_IDENT_DIGITS * 2) - 3);
+    wmove(statusbar->topline, 0, x2 - KEY_IDENT_BYTES  - 3);
 
     wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
     wprintw(statusbar->topline, "{");
@@ -1425,7 +1475,7 @@ static void chat_onDraw(ToxWindow *self, Tox *m)
 
     wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
 
-    for (size_t i = 0; i < KEY_IDENT_DIGITS; ++i) {
+    for (size_t i = 0; i < KEY_IDENT_BYTES / 2; ++i) {
         wprintw(statusbar->topline, "%02X", Friends.list[self->num].pub_key[i] & 0xff);
     }
 
@@ -1552,6 +1602,8 @@ static void chat_onInit(ToxWindow *self, Tox *m)
 
     scrollok(ctx->history, 0);
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
+
+    line_info_print(self);
 }
 
 ToxWindow *new_chat(Tox *m, uint32_t friendnum)
@@ -1579,6 +1631,7 @@ ToxWindow *new_chat(Tox *m, uint32_t friendnum)
     ret->onFileControl = &chat_onFileControl;
     ret->onFileRecv = &chat_onFileRecv;
     ret->onReadReceipt = &chat_onReadReceipt;
+    ret->onGroupInvite = &chat_onGroupInvite;
 
 #ifdef AUDIO
     ret->onInvite = &chat_onInvite;
