@@ -29,6 +29,7 @@
 #include "file_transfers.h"
 #include "friendlist.h"
 #include "line_info.h"
+#include "groupchats.h"
 #include "misc_tools.h"
 #include "toxic.h"
 #include "windows.h"
@@ -36,6 +37,40 @@
 extern ToxWindow *prompt;
 extern FriendsList Friends;
 
+void cmd_autoaccept_files(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+{
+    UNUSED_VAR(window);
+    UNUSED_VAR(m);
+
+    const char *msg;
+    const bool auto_accept_files = friend_get_auto_accept_files(self->num);
+
+    if (argc == 0) {
+        if (auto_accept_files) {
+            msg = "Auto-file accept for this friend is enabled; type \"/autoaccept off\" to disable";
+        } else {
+            msg = "Auto-file accept for this friend is disabled; type \"/autoaccept on\" to enable";
+        }
+
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+        return;
+    }
+
+    const char *option = argv[1];
+
+    if (!strcmp(option, "1") || !strcmp(option, "on")) {
+        friend_set_auto_file_accept(self->num, true);
+        msg = "Auto-accepting file transfers has been enabled for this friend";
+    } else if (!strcmp(option, "0") || !strcmp(option, "off")) {
+        friend_set_auto_file_accept(self->num, false);
+        msg = "Auto-accepting file transfers has been disabled for this friend";
+    } else {
+        msg = "Invalid option. Use \"/autoaccept on\" and \"/autoaccept off\" to toggle auto-file accept";
+    }
+
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+}
+
 void cmd_cancelfile(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
@@ -178,6 +213,77 @@ void cmd_conference_join(WINDOW *window, ToxWindow *self, Tox *m, int argc, char
 #endif
 }
 
+void cmd_group_accept(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+{
+    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
+        return;
+    }
+
+    if (Friends.list[self->num].group_invite.length == 0) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending group invite");
+        return;
+    }
+
+    const char *passwd = NULL;
+    uint16_t passwd_len = 0;
+
+    if (argc > 0) {
+        passwd = argv[1];
+        passwd_len = strlen(passwd);
+    }
+
+    size_t nick_len = tox_self_get_name_size(m);
+    char self_nick[TOX_MAX_NAME_LENGTH + 1];
+    tox_self_get_name(m, (uint8_t *) self_nick);
+    self_nick[nick_len] = '\0';
+
+    Tox_Err_Group_Invite_Accept err;
+    uint32_t groupnumber = tox_group_invite_accept(m, self->num, Friends.list[self->num].group_invite.data,
+                           Friends.list[self->num].group_invite.length, (const uint8_t *) self_nick, nick_len,
+                           (const uint8_t *) passwd, passwd_len, &err);
+
+    if (err != TOX_ERR_GROUP_INVITE_ACCEPT_OK) {
+        if (err == TOX_ERR_GROUP_INVITE_ACCEPT_TOO_LONG) {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to join group: Password too long.");
+        } else {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to join group (error %d).", err);
+        }
+
+        return;
+    }
+
+    if (init_groupchat_win(m, groupnumber, NULL, 0, Group_Join_Type_Join) == -1) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Group chat window failed to initialize.");
+        tox_group_leave(m, groupnumber, NULL, 0, NULL);
+        return;
+    }
+}
+
+void cmd_group_invite(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+{
+    if (argc < 1) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Group number required.");
+        return;
+    }
+
+    int groupnumber = atoi(argv[1]);
+
+    if (groupnumber == 0 && strcmp(argv[1], "0")) {    /* atoi returns 0 value on invalid input */
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid group number.");
+        return;
+    }
+
+    Tox_Err_Group_Invite_Friend err;
+
+    if (!tox_group_invite_friend(m, groupnumber, self->num, &err)) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to invite contact to group (error %d).", err);
+        return;
+    }
+
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Invited contact to Group %d.", groupnumber);
+}
+
 #ifdef GAMES
 
 void cmd_game_join(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
@@ -200,7 +306,7 @@ void cmd_game_join(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*arg
     uint8_t *data = Friends.list[self->num].game_invite.data;
     size_t length = Friends.list[self->num].game_invite.data_length;
 
-    int ret = game_initialize(self, m, type, id, data, length);
+    int ret = game_initialize(self, m, type, id, data, length, false);
 
     switch (ret) {
         case 0: {
@@ -241,19 +347,19 @@ void cmd_savefile(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv
     long int idx = strtol(argv[1], NULL, 10);
 
     if ((idx == 0 && strcmp(argv[1], "0")) || idx < 0 || idx >= MAX_FILES) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with that ID.");
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with ID %ld", idx);
         return;
     }
 
     FileTransfer *ft = get_file_transfer_struct_index(self->num, idx, FILE_TRANSFER_RECV);
 
     if (!ft) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with that ID.");
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with ID %ld.", idx);
         return;
     }
 
     if (ft->state != FILE_TRANSFER_PENDING) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with that ID.");
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with ID %ld.", idx);
         return;
     }
 
@@ -272,12 +378,13 @@ void cmd_savefile(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv
 
     line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Saving file [%ld] as: '%s'", idx, ft->file_path);
 
-    /* prep progress bar line */
+    const bool auto_accept_files = friend_get_auto_accept_files(self->num);
+    const uint32_t line_skip = auto_accept_files ? 4 : 2;
+
     char progline[MAX_STR_SIZE];
     init_progress_bar(progline);
     line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s", progline);
-
-    ft->line_id = self->chatwin->hst->line_end->id + 2;
+    ft->line_id = self->chatwin->hst->line_end->id + line_skip;
     ft->state = FILE_TRANSFER_STARTED;
 
     return;
