@@ -24,7 +24,6 @@
 #define _GNU_SOURCE    /* needed for strcasestr() and wcswidth() */
 #endif
 
-#include <assert.h>
 #include <inttypes.h>
 #include <math.h>
 #include <stdlib.h>
@@ -85,11 +84,13 @@ static const char *conference_cmd_list[] = {
     "/connect",
     "/decline",
     "/exit",
+    "/group",
     "/conference",
 #ifdef GAMES
     "/game",
 #endif
     "/help",
+    "/join",
     "/log",
 #ifdef AUDIO
     "/mute",
@@ -140,15 +141,22 @@ void conference_set_title(ToxWindow *self, uint32_t conferencesnum, const char *
 
 static void kill_conference_window(ToxWindow *self)
 {
+    if (self == NULL) {
+        return;
+    }
+
     ChatContext *ctx = self->chatwin;
 
-    log_disable(ctx->log);
-    line_info_cleanup(ctx->hst);
-    delwin(ctx->linewin);
-    delwin(ctx->history);
-    delwin(ctx->sidebar);
-    free(ctx->log);
-    free(ctx);
+    if (ctx != NULL) {
+        log_disable(ctx->log);
+        line_info_cleanup(ctx->hst);
+        delwin(ctx->linewin);
+        delwin(ctx->history);
+        delwin(ctx->sidebar);
+        free(ctx->log);
+        free(ctx);
+    }
+
     free(self->help);
     kill_notifs(self->active_box);
     del_window(self);
@@ -1339,7 +1347,13 @@ bool enable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
         }
     }
 
-    bool success = init_conference_audio_input(tox, conferencenum);
+    const ConferenceChat *chat = &conferences[conferencenum];
+
+    if (chat->audio_enabled) {
+        return true;
+    }
+
+    const bool success = init_conference_audio_input(tox, conferencenum);
 
     if (success) {
         self->is_call = true;
@@ -1359,9 +1373,11 @@ bool disable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
     if (chat->audio_enabled) {
         close_device(input, chat->audio_in_idx);
         chat->audio_enabled = false;
+    } else {
+        return true;
     }
 
-    bool success = toxav_groupchat_disable_av(tox, conferencenum) == 0;
+    const bool success = toxav_groupchat_disable_av(tox, conferencenum) == 0;
 
     if (success) {
         self->is_call = false;
