@@ -20,7 +20,6 @@
  *
  */
 
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -29,6 +28,7 @@
 #include "execute.h"
 #include "global_commands.h"
 #include "conference_commands.h"
+#include "groupchat_commands.h"
 #include "line_info.h"
 #include "misc_tools.h"
 #include "notify.h"
@@ -49,10 +49,12 @@ static struct cmd_func global_commands[] = {
     { "/decline",   cmd_decline       },
     { "/exit",      cmd_quit          },
     { "/conference", cmd_conference    },
+    { "/group",     cmd_groupchat     },
 #ifdef GAMES
     { "/game",      cmd_game          },
 #endif
     { "/help",      cmd_prompt_help   },
+    { "/join",      cmd_join          },
     { "/log",       cmd_log           },
     { "/myid",      cmd_myid          },
 #ifdef QRCODE
@@ -80,9 +82,12 @@ static struct cmd_func global_commands[] = {
 };
 
 static struct cmd_func chat_commands[] = {
+    { "/autoaccept", cmd_autoaccept_files  },
     { "/cancel",    cmd_cancelfile        },
-    { "/invite",    cmd_conference_invite },
-    { "/join",      cmd_conference_join   },
+    { "/cinvite",   cmd_conference_invite },
+    { "/cjoin",     cmd_conference_join   },
+    { "/gaccept",   cmd_group_accept      },
+    { "/invite",    cmd_group_invite      },
 #ifdef GAMES
     { "/play",      cmd_game_join         },
 #endif
@@ -117,12 +122,47 @@ static struct cmd_func conference_commands[] = {
     { NULL,         NULL             },
 };
 
+static struct cmd_func groupchat_commands[] = {
+    { "/chatid",    cmd_chatid         },
+    { "/disconnect", cmd_disconnect     },
+    { "/ignore",    cmd_ignore         },
+    { "/kick",      cmd_kick           },
+    { "/list",      cmd_list           },
+    { "/locktopic", cmd_set_topic_lock },
+    { "/mod",       cmd_mod            },
+    { "/nick",      cmd_group_nick     },
+    { "/passwd",    cmd_set_passwd     },
+    { "/peerlimit", cmd_set_peerlimit  },
+    { "/privacy",   cmd_set_privacy    },
+    { "/rejoin",    cmd_rejoin         },
+    { "/silence",   cmd_silence        },
+    { "/topic",     cmd_set_topic      },
+    { "/unignore",  cmd_unignore       },
+    { "/unmod",     cmd_unmod          },
+    { "/unsilence", cmd_unsilence      },
+    { "/voice",     cmd_set_voice      },
+    { "/whois",     cmd_whois          },
+    { NULL,         NULL               },
+};
+
 /* Special commands are commands that only take one argument even if it contains spaces */
 static const char special_commands[][MAX_CMDNAME_SIZE] = {
     "/add",
     "/avatar",
+    "/gaccept",
+    "/group",
+    "/ignore",
+    "/kick",
+    "/mod",
     "/nick",
     "/note",
+    "/passwd",
+    "/silence",
+    "/topic",
+    "/unignore",
+    "/unmod",
+    "/unsilence",
+    "/whois",
 #ifdef PYTHON
     "/run",
 #endif /* PYTHON */
@@ -243,19 +283,29 @@ void execute(WINDOW *w, ToxWindow *self, Tox *m, const char *input, int mode)
      * Note: Global commands must come last in case of duplicate command names
      */
     switch (mode) {
-        case CHAT_COMMAND_MODE:
+        case CHAT_COMMAND_MODE: {
             if (do_command(w, self, m, num_args, chat_commands, args) == 0) {
                 return;
             }
 
             break;
+        }
 
-        case CONFERENCE_COMMAND_MODE:
+        case CONFERENCE_COMMAND_MODE: {
             if (do_command(w, self, m, num_args, conference_commands, args) == 0) {
                 return;
             }
 
             break;
+        }
+
+        case GROUPCHAT_COMMAND_MODE: {
+            if (do_command(w, self, m, num_args, groupchat_commands, args) == 0) {
+                return;
+            }
+
+            break;
+        }
     }
 
     if (do_command(w, self, m, num_args, global_commands, args) == 0) {
