@@ -21,7 +21,6 @@
  */
 
 #include <arpa/inet.h>
-#include <assert.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -116,6 +115,10 @@ void kill_friendlist(ToxWindow *self)
             free(Friends.list[i].game_invite.data);
 #endif
         }
+
+        if (Friends.list[i].group_invite.data != NULL) {
+            free(Friends.list[i].group_invite.data);
+        }
     }
 
     realloc_blocklist(0);
@@ -498,6 +501,7 @@ void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
         Friends.list[i].num = num;
         Friends.list[i].active = true;
         Friends.list[i].chatwin = -1;
+        Friends.list[i].auto_accept_files = false;  // do not change
         Friends.list[i].connection_status = TOX_CONNECTION_NONE;
         Friends.list[i].status = TOX_USER_STATUS_NONE;
         Friends.list[i].logging_on = (bool) user_settings->autolog == AUTOLOG_ON;
@@ -674,6 +678,35 @@ static void friendlist_onConferenceInvite(ToxWindow *self, Tox *m, int32_t num,
     sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
 }
 
+static void friendlist_onGroupInvite(ToxWindow *self, Tox *m, uint32_t num, const char *data, size_t length,
+                                     const char *group_name, size_t group_name_length)
+{
+    UNUSED_VAR(self);
+    UNUSED_VAR(data);
+    UNUSED_VAR(length);
+
+    if (num >= Friends.max_idx) {
+        return;
+    }
+
+    if (Friends.list[num].chatwin != -1) {
+        return;
+    }
+
+    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
+        Friends.list[num].chatwin = add_window(m, new_chat(m, Friends.list[num].num));
+        return;
+    }
+
+    char nick[TOX_MAX_NAME_LENGTH];
+    get_nick_truncate(m, nick, num);
+
+    line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED,
+                  "* Group chat invite from %s failed: too many windows are open.", nick);
+
+    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+}
+
 /* move friendlist/blocklist cursor up and down */
 static void select_friend(wint_t key, int *selected, int num)
 {
@@ -1311,6 +1344,10 @@ void friendlist_onInit(ToxWindow *self, Tox *m)
 
 void disable_chatwin(uint32_t f_num)
 {
+    if (f_num >= Friends.max_idx) {
+        return;
+    }
+
     Friends.list[f_num].chatwin = -1;
 }
 
@@ -1348,12 +1385,20 @@ static void friendlist_onAV(ToxWindow *self, ToxAV *av, uint32_t friend_number,
 /* Returns a friend's status */
 Tox_User_Status get_friend_status(uint32_t friendnumber)
 {
+    if (friendnumber >= Friends.max_idx) {
+        return TOX_USER_STATUS_NONE;
+    }
+
     return Friends.list[friendnumber].status;
 }
 
 /* Returns a friend's connection status */
 Tox_Connection get_friend_connection_status(uint32_t friendnumber)
 {
+    if (friendnumber >= Friends.max_idx) {
+        return TOX_CONNECTION_NONE;
+    }
+
     return Friends.list[friendnumber].connection_status;
 }
 
@@ -1377,6 +1422,24 @@ bool friend_is_blocked(const char *public_key)
     return false;
 }
 
+void friend_set_auto_file_accept(uint32_t friendnumber, bool auto_accept)
+{
+    if (friendnumber >= Friends.max_idx) {
+        return;
+    }
+
+    Friends.list[friendnumber].auto_accept_files = auto_accept;
+}
+
+bool friend_get_auto_accept_files(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        return false;
+    }
+
+    return Friends.list[friendnumber].auto_accept_files;
+}
+
 ToxWindow *new_friendlist(void)
 {
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
@@ -1398,6 +1461,7 @@ ToxWindow *new_friendlist(void)
     ret->onStatusMessageChange = &friendlist_onStatusMessageChange;
     ret->onFileRecv = &friendlist_onFileRecv;
     ret->onConferenceInvite = &friendlist_onConferenceInvite;
+    ret->onGroupInvite = &friendlist_onGroupInvite;
 
 #ifdef AUDIO
     ret->onInvite = &friendlist_onAV;
