@@ -46,6 +46,11 @@ struct ConferenceInvite {
     bool pending;
 };
 
+struct GroupInvite {
+    uint8_t *data;
+    uint16_t length;
+};
+
 #ifdef GAMES
 
 struct GameInvite {
@@ -70,17 +75,20 @@ typedef struct {
     Tox_Connection connection_status;
     bool is_typing;
     bool logging_on;    /* saves preference for friend irrespective of global settings */
+    bool auto_accept_files;  /* default should always be false */
     Tox_User_Status status;
 
     struct LastOnline last_online;
-    struct ConferenceInvite conference_invite;
 
 #ifdef GAMES
     struct GameInvite game_invite;
 #endif
 
-    FileTransfer file_receiver[MAX_FILES];
-    FileTransfer file_sender[MAX_FILES];
+    struct ConferenceInvite conference_invite;
+    struct GroupInvite group_invite;
+
+    struct FileTransfer file_receiver[MAX_FILES];
+    struct FileTransfer file_sender[MAX_FILES];
     PendingFileTransfer file_send_queue[MAX_FILES];
 } ToxicFriend;
 
@@ -124,4 +132,14 @@ void sort_friendlist_index(void);
  */
 bool friend_is_blocked(const char *public_key);
 
+/*
+ * Enable or disable auto-accepting file transfers for this friend.
+ */
+void friend_set_auto_file_accept(uint32_t friendnumber, bool auto_accept);
+
+/*
+ * Return true if auto-accepting file transfers is enabled for this friend.
+ */
+bool friend_get_auto_accept_files(uint32_t friendnumber);
+
 #endif /* end of include guard: FRIENDLIST_H */
