@@ -123,11 +123,25 @@ void game_list_print(ToxWindow *self)
     }
 }
 
-bool game_type_is_multiplayer(GameType type)
+bool game_type_has_multiplayer(GameType type)
+{
+    return type == GT_Chess || type == GT_Snake;
+}
+
+static bool game_type_is_multi_only(GameType type)
 {
     return type == GT_Chess;
 }
 
+static bool game_type_is_multi_and_single(const ToxWindow *window, GameType type)
+{
+    if (window->type != WINDOW_TYPE_CHAT) {
+        return false;
+    }
+
+    return type == GT_Snake;
+}
+
 /*
  * Sends a notification to the window associated with `game`.
  *
@@ -203,13 +217,13 @@ static void game_toggle_pause(GameData *game)
     }
 }
 
-static int game_initialize_type(GameData *game, const uint8_t *data, size_t length)
+static int game_initialize_type(GameData *game, const uint8_t *data, size_t length, bool self_host)
 {
     int ret = -3;
 
     switch (game->type) {
         case GT_Snake: {
-            ret = snake_initialize(game);
+            ret = snake_initialize(game, game->is_multiplayer, self_host);
             break;
         }
 
@@ -219,7 +233,7 @@ static int game_initialize_type(GameData *game, const uint8_t *data, size_t leng
         }
 
         case GT_Chess: {
-            ret = chess_initialize(game, data, length);
+            ret = chess_initialize(game, data, length, self_host);
             break;
         }
 
@@ -237,7 +251,7 @@ static int game_initialize_type(GameData *game, const uint8_t *data, size_t leng
 }
 
 int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id, const uint8_t *multiplayer_data,
-                    size_t length)
+                    size_t length, bool self_host)
 {
     int max_x;
     int max_y;
@@ -261,7 +275,7 @@ int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id,
         return -4;
     }
 
-    game->is_multiplayer = game_type_is_multiplayer(type);
+    game->is_multiplayer = game_type_is_multi_only(type) || game_type_is_multi_and_single(parent, type);
 
     if (game->is_multiplayer) {
         if (parent->type != WINDOW_TYPE_CHAT) {
@@ -293,7 +307,7 @@ int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id,
         return -4;
     }
 
-    int init_ret = game_initialize_type(game, multiplayer_data, length);
+    const int init_ret = game_initialize_type(game, multiplayer_data, length, self_host);
 
     if (init_ret < 0) {
         game_init_abort(parent, self);
@@ -461,6 +475,14 @@ int game_set_message(GameData *game, const char *message, size_t length, Directi
         return -1;
     }
 
+    int max_x;
+    int max_y;
+    getmaxyx(game->window, max_y, max_x);
+
+    if (coords->x > max_x || coords->x < 0 || coords->y > max_y || coords->y < 0) {
+        return -1;
+    }
+
     GameMessage *m = game_get_new_message_holder(game);
 
     if (m == NULL) {
@@ -506,7 +528,7 @@ static int game_restart(GameData *game)
 
     game_clear_all_messages(game);
 
-    if (game_initialize_type(game, NULL, 0) == -1) {
+    if (game_initialize_type(game, NULL, 0, false) == -1) {
         return -1;
     }
 
@@ -603,17 +625,28 @@ static void game_draw_game_over(const GameData *game)
     const int x = max_x / 2;
     const int y = max_y / 2;
 
-    const char *message = "GAME OVER!";
+    const char *message;
+    int colour = RED;
+
+    if (game->is_multiplayer) {
+        message = game->winner ? "You win!" : "You lose!";
+        colour = game->winner ? YELLOW : RED;
+    } else {
+        message = "GAME OVER!";
+    }
+
     size_t length = strlen(message);
 
-    wattron(win, A_BOLD | COLOR_PAIR(RED));
+    wattron(win, A_BOLD | COLOR_PAIR(colour));
     mvwprintw(win, y - 1, x - (length / 2), "%s", message);
-    wattroff(win, A_BOLD | COLOR_PAIR(RED));
+    wattroff(win, A_BOLD | COLOR_PAIR(colour));
 
-    message = "Press F5 to play again";
-    length = strlen(message);
+    if (!game->is_multiplayer) {
+        message = "Press F5 to play again";
+        length = strlen(message);
 
-    mvwprintw(win, y + 1, x - (length / 2), "%s", message);
+        mvwprintw(win, y + 1, x - (length / 2), "%s", message);
+    }
 }
 
 static void game_draw_pause_screen(const GameData *game)
@@ -752,7 +785,9 @@ bool game_onKey(ToxWindow *self, Tox *m, wint_t key, bool is_printable)
     GameData *game = self->game;
 
     if (key == KEY_F(9)) {
+        pthread_mutex_lock(&Winthread.lock);
         game_kill(self);
+        pthread_mutex_unlock(&Winthread.lock);
         return true;
     }
 
@@ -881,11 +916,18 @@ static ToxWindow *game_new_window(Tox *m, GameType type, uint32_t friendnumber)
 
     ret->active_box = -1;
 
-    if (game_type_is_multiplayer(type)) {
+    if (game_type_is_multi_only(type)) {
         char nick[TOX_MAX_NAME_LENGTH];
         get_nick_truncate(m, nick, friendnumber);
 
-        snprintf(ret->name, sizeof(ret->name), "%s (%s)", window_name, nick);
+        char buf[sizeof(nick) + sizeof(ret->name) + 4];
+        snprintf(buf, sizeof(buf), "%s (%s)", window_name, nick);
+
+        const size_t name_size = sizeof(ret->name);
+
+        buf[name_size - 1] = '\0';
+
+        snprintf(ret->name, name_size, "%s", buf);
     } else {
         snprintf(ret->name, sizeof(ret->name), "%s", window_name);
     }
@@ -924,8 +966,8 @@ void game_random_coords(const GameData *game, Coords *coords)
     const int y_top_bound    = ((max_y - game_max_y) / 2) + 1;
     const int y_bottom_bound = ((max_y + game_max_y) / 2) - 1;
 
-    coords->x = (rand() % (x_right_bound - x_left_bound + 1)) + x_left_bound;
-    coords->y = (rand() % (y_bottom_bound - y_top_bound + 1)) + y_top_bound;
+    coords->x = (int)rand_range_not_secure(x_right_bound - x_left_bound + 1) + x_left_bound;
+    coords->y = (int)rand_range_not_secure(y_bottom_bound - y_top_bound + 1) + y_top_bound;
 }
 
 void game_max_x_y(const GameData *game, int *x, int *y)
@@ -1043,6 +1085,13 @@ void game_set_status(GameData *game, GameStatus status)
     }
 }
 
+void game_set_winner(GameData *game, bool winner)
+{
+    if (game->status == GS_Finished) {
+        game->winner = winner;
+    }
+}
+
 void game_set_update_interval(GameData *game, TIME_MS update_interval)
 {
     game->update_interval = MIN(update_interval, GAME_MAX_UPDATE_INTERVAL);
@@ -1129,7 +1178,7 @@ int game_packet_send(const GameData *game, const uint8_t *data, size_t length, G
     memcpy(packet + 1 + GAME_PACKET_HEADER_SIZE, data, length);
     packet_length += length;
 
-    TOX_ERR_FRIEND_CUSTOM_PACKET err;
+    Tox_Err_Friend_Custom_Packet err;
 
     if (!tox_friend_send_lossless_packet(game->tox, game->friend_number, packet, packet_length, &err)) {
         fprintf(stderr, "failed to send game packet: error %d\n", err);
