@@ -20,12 +20,17 @@
  *
  */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE    /* needed for wcswidth() */
+#endif
+
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "conference.h"
+#include "groupchats.h"
 #include "line_info.h"
 #include "message_queue.h"
 #include "misc_tools.h"
@@ -80,6 +85,10 @@ void line_info_reset_start(ToxWindow *self, struct history *hst)
 
 void line_info_cleanup(struct history *hst)
 {
+    if (hst == NULL) {
+        return;
+    }
+
     struct line_info *tmp1 = hst->line_root;
 
     while (tmp1) {
@@ -137,13 +146,16 @@ static struct line_info *line_info_ret_queue(struct history *hst)
  * Return 0 if string does not contain a newline byte.
  * Return -1 if printing was aborted.
  */
-static int print_n_chars(WINDOW *win, const char *s, size_t n, int max_y)
+static int print_n_chars(WINDOW *win, const wchar_t *s, size_t n, int max_y)
 {
+    // we use an array to represent a single wchar in order to get around an ncurses
+    // bug with waddnwstr() that overreads the memory address by one byte when
+    // supplied with a single wchar.
+    wchar_t ch[2] = {0};
     bool newline = false;
-    char ch;
 
-    for (size_t i = 0; i < n && (ch = s[i]); ++i) {
-        if (ch == '\n') {
+    for (size_t i = 0; i < n && (ch[0] = s[i]); ++i) {
+        if (ch[0] == L'\n') {
             newline = true;
 
             int x;
@@ -159,7 +171,17 @@ static int print_n_chars(WINDOW *win, const char *s, size_t n, int max_y)
         }
 
         if (win) {
-            wprintw(win, "%c", ch);
+#ifdef HAVE_WIDECHAR
+            waddnwstr(win, ch, 1);
+#else
+            char b;
+
+            if (wcstombs(&b, ch, sizeof(char)) != 1) {
+                continue;
+            }
+
+            wprintw(win, "%c", b);
+#endif  // HAVE_WIDECHAR
         }
     }
 
@@ -169,10 +191,10 @@ static int print_n_chars(WINDOW *win, const char *s, size_t n, int max_y)
 /* Returns the index of the last space character in `s` found before `limit`.
  * Returns -1 if no space is found.
  */
-static int rspace_index(const char *s, int limit)
+static int rspace_index(const wchar_t *s, int limit)
 {
     for (int i = limit; i >= 0; --i) {
-        if (s[i] == ' ') {
+        if (s[i] == L' ') {
             return i;
         }
     }
@@ -183,12 +205,12 @@ static int rspace_index(const char *s, int limit)
 /* Returns the first index in `s` containing a newline byte found before `limit`.
  * Returns -1 if no newline  is found.
  */
-static int newline_index(const char *s, int limit)
+static int newline_index(const wchar_t *s, int limit)
 {
-    char ch;
+    wchar_t ch;
 
     for (int i = 0; i < limit && (ch = s[i]); ++i) {
-        if (ch == '\n') {
+        if (ch == L'\n') {
             return i;
         }
     }
@@ -197,13 +219,13 @@ static int newline_index(const char *s, int limit)
 }
 
 /* Returns the number of newline bytes in `s` */
-static unsigned int newline_count(const char *s)
+static unsigned int newline_count(const wchar_t *s)
 {
-    char ch;
+    wchar_t ch;
     unsigned int count = 0;
 
     for (size_t i = 0; (ch = s[i]); ++i) {
-        if (ch == '\n') {
+        if (ch == L'\n') {
             ++count;
         }
     }
@@ -226,10 +248,10 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
     int y;
     UNUSED_VAR(y);
 
-    const char *msg = line->msg;
-    uint16_t length = line->msg_len;
+    const wchar_t *msg = line->msg;
+    uint16_t length = line->msg_width;
     uint16_t lines = 0;
-    const int x_start = line->len - line->msg_len - 1;  // manually keep track of x position because ncurses sucks
+    const int x_start = line->len - line->msg_width - 1;  // manually keep track of x position because ncurses sucks
     int x_limit = max_x - x_start;
 
     if (x_limit <= 1) {
@@ -246,7 +268,7 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
         }
 
         if (length < x_limit) {
-            int p_ret = print_n_chars(win, msg, length, max_y);
+            const int p_ret = print_n_chars(win, msg, length, max_y);
 
             if (p_ret == 1) {
                 lines += newline_count(msg);
@@ -258,7 +280,7 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
             break;
         }
 
-        int newline_idx = newline_index(msg, x_limit - 1);
+        const int newline_idx = newline_index(msg, x_limit - 1);
 
         if (newline_idx >= 0) {
             if (print_n_chars(win, msg, newline_idx + 1, max_y) == -1) {
@@ -272,7 +294,7 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
             continue;
         }
 
-        int space_idx = rspace_index(msg, x_limit - 1);
+        const int space_idx = rspace_index(msg, x_limit - 1);
 
         if (space_idx >= 1) {
             if (print_n_chars(win, msg, space_idx, max_y) == -1) {
@@ -321,6 +343,40 @@ static int print_wrap(WINDOW *win, struct line_info *line, int max_x, int max_y)
     return 0;
 }
 
+/* Converts the multibyte string `msg` into a wide character string and puts
+ * the result in `buf`.
+ *
+ * `buf_size` is the number of wide characters that `buf` can hold.
+ *
+ * Returns the widechar width of the string.
+ */
+static uint16_t line_info_add_msg(wchar_t *buf, size_t buf_size, const char *msg)
+{
+    if (msg == NULL || msg[0] == '\0') {
+        return 0;
+    }
+
+    const int wc_msg_len = mbs_to_wcs_buf(buf, msg, buf_size);
+
+    if (wc_msg_len > 0 && wc_msg_len < buf_size) {
+        buf[wc_msg_len] = L'\0';
+        int width = wcswidth(buf, wc_msg_len);
+
+        if (width < 0 || width > UINT16_MAX) {  // the best we can do on failure is to fall back to strlen
+            width = strlen(msg);
+        }
+
+        return (uint16_t)width;
+    } else {
+        fprintf(stderr, "Failed to convert string '%s' to widechar\n", msg);
+        const wchar_t *err = L"Failed to parse message";
+        uint16_t width = (uint16_t)wcslen(err);
+        wmemcpy(buf, err, width);
+        buf[width] = L'\0';
+        return width;
+    }
+}
+
 static void line_info_init_line(ToxWindow *self, struct line_info *line)
 {
     int y2;
@@ -373,18 +429,29 @@ int line_info_add(ToxWindow *self, bool show_timestamp, const char *name1, const
     switch (type) {
         case IN_ACTION:
 
+        /* fallthrough */
+        case OUT_ACTION_READ:
+
         /* fallthrough */
         case OUT_ACTION:
             len += strlen(user_settings->line_normal) + 2; // two spaces
             break;
 
         case IN_MSG:
+        case OUT_MSG_READ:
 
         /* fallthrough */
         case OUT_MSG:
             len += strlen(user_settings->line_normal) + 3; // two spaces and a ':' char
             break;
 
+        case IN_PRVT_MSG:
+
+        /* fallthrough */
+        case OUT_PRVT_MSG:
+            len += strlen(user_settings->line_special) + 3;
+            break;
+
         case CONNECTION:
             len += strlen(user_settings->line_join) + 2;  // two spaces
             break;
@@ -409,13 +476,8 @@ int line_info_add(ToxWindow *self, bool show_timestamp, const char *name1, const
             break;
     }
 
-    uint16_t msg_len = 0;
-
-    if (frmt_msg[0]) {
-        snprintf(new_line->msg, sizeof(new_line->msg), "%s", frmt_msg);
-        msg_len = strlen(new_line->msg);
-        len += msg_len;
-    }
+    const uint16_t msg_width = line_info_add_msg(new_line->msg, sizeof(new_line->msg) / sizeof(wchar_t), frmt_msg);
+    len += msg_width;
 
     if (show_timestamp) {
         get_time_str(new_line->timestr, sizeof(new_line->timestr));
@@ -434,7 +496,7 @@ int line_info_add(ToxWindow *self, bool show_timestamp, const char *name1, const
 
     new_line->id = (hst->line_end->id + 1 + hst->queue_size) % INT_MAX;
     new_line->len = len;
-    new_line->msg_len = msg_len;
+    new_line->msg_width = msg_width;
     new_line->type = type;
     new_line->bold = bold;
     new_line->colour = colour;
@@ -503,7 +565,7 @@ void line_info_print(ToxWindow *self)
         return;
     }
 
-    if (self->type == WINDOW_TYPE_CONFERENCE) {
+    if (self->type == WINDOW_TYPE_CONFERENCE || self->type == WINDOW_TYPE_GROUPCHAT) {
         wmove(win, 0, 0);
     } else {
         wmove(win, TOP_BAR_HEIGHT, 0);
@@ -518,9 +580,8 @@ void line_info_print(ToxWindow *self)
     const int max_y = y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT;
     const int max_x = self->show_peerlist ? x2 - 1 - SIDEBAR_WIDTH : x2;
     uint16_t numlines = line->format_lines;
-    int print_ret = 0;
 
-    while (line && numlines++ <= max_y && print_ret == 0) {
+    while (line && numlines++ <= max_y) {
         int y;
         int x;
         getyx(win, y, x);
@@ -540,7 +601,7 @@ void line_info_print(ToxWindow *self)
             case OUT_MSG_READ:
 
             /* fallthrough */
-            case IN_MSG:
+            case IN_MSG: {
                 wattron(win, COLOR_PAIR(BLUE));
                 wprintw(win, "%s ", line->timestr);
                 wattroff(win, COLOR_PAIR(BLUE));
@@ -557,18 +618,51 @@ void line_info_print(ToxWindow *self)
                 wprintw(win, "%s %s: ", user_settings->line_normal, line->name1);
                 wattroff(win, COLOR_PAIR(nameclr));
 
-                if (line->msg[0] == 0) {
+                if (line->msg[0] == L'\0') {
                     waddch(win, '\n');
                     break;
                 }
 
+                if (line->msg[0] == L'>') {
+                    wattron(win, COLOR_PAIR(GREEN));
+                } else if (line->msg[0] == L'<') {
+                    wattron(win, COLOR_PAIR(RED));
+                }
+
+                print_wrap(win, line, max_x, max_y);
+
+                if (line->msg[0] == L'>') {
+                    wattroff(win, COLOR_PAIR(GREEN));
+                } else if (line->msg[0] == L'<') {
+                    wattroff(win, COLOR_PAIR(RED));
+                }
+
+                waddch(win, '\n');
+                break;
+            }
+
+            case IN_PRVT_MSG:
+
+            /* fallthrough */
+
+            case OUT_PRVT_MSG: {
+                wattron(win, COLOR_PAIR(BLUE));
+                wprintw(win, "%s ", line->timestr);
+                wattroff(win, COLOR_PAIR(BLUE));
+
+                const int nameclr = line->colour ? line->colour : GREEN;
+
+                wattron(win, COLOR_PAIR(nameclr));
+                wprintw(win, "%s %s: ", user_settings->line_special, line->name1);
+                wattroff(win, COLOR_PAIR(nameclr));
+
                 if (line->msg[0] == '>') {
                     wattron(win, COLOR_PAIR(GREEN));
                 } else if (line->msg[0] == '<') {
                     wattron(win, COLOR_PAIR(RED));
                 }
 
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
 
                 if (line->msg[0] == '>') {
                     wattroff(win, COLOR_PAIR(GREEN));
@@ -578,6 +672,7 @@ void line_info_print(ToxWindow *self)
 
                 waddch(win, '\n');
                 break;
+            }
 
             case OUT_ACTION_READ:
 
@@ -585,20 +680,21 @@ void line_info_print(ToxWindow *self)
             case OUT_ACTION:
 
             /* fallthrough */
-            case IN_ACTION:
+            case IN_ACTION: {
                 wattron(win, COLOR_PAIR(BLUE));
                 wprintw(win, "%s ", line->timestr);
                 wattroff(win, COLOR_PAIR(BLUE));
 
                 wattron(win, COLOR_PAIR(YELLOW));
                 wprintw(win, "%s %s ", user_settings->line_normal, line->name1);
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
                 wattroff(win, COLOR_PAIR(YELLOW));
 
                 waddch(win, '\n');
                 break;
+            }
 
-            case SYS_MSG:
+            case SYS_MSG: {
                 if (line->timestr[0]) {
                     wattron(win, COLOR_PAIR(BLUE));
                     wprintw(win, "%s ", line->timestr);
@@ -613,7 +709,7 @@ void line_info_print(ToxWindow *self)
                     wattron(win, COLOR_PAIR(line->colour));
                 }
 
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
                 waddch(win, '\n');
 
                 if (line->bold) {
@@ -625,20 +721,22 @@ void line_info_print(ToxWindow *self)
                 }
 
                 break;
+            }
 
-            case PROMPT:
+            case PROMPT: {
                 wattron(win, COLOR_PAIR(GREEN));
                 wprintw(win, "$ ");
                 wattroff(win, COLOR_PAIR(GREEN));
 
-                if (line->msg[0]) {
-                    print_ret = print_wrap(win, line, max_x, max_y);
+                if (line->msg[0] != L'\0') {
+                    print_wrap(win, line, max_x, max_y);
                 }
 
                 waddch(win, '\n');
                 break;
+            }
 
-            case CONNECTION:
+            case CONNECTION: {
                 wattron(win, COLOR_PAIR(BLUE));
                 wprintw(win, "%s ", line->timestr);
                 wattroff(win, COLOR_PAIR(BLUE));
@@ -650,14 +748,15 @@ void line_info_print(ToxWindow *self)
                 wprintw(win, "%s ", line->name1);
                 wattroff(win, A_BOLD);
 
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
                 waddch(win, '\n');
 
                 wattroff(win, COLOR_PAIR(line->colour));
 
                 break;
+            }
 
-            case DISCONNECTION:
+            case DISCONNECTION: {
                 wattron(win, COLOR_PAIR(BLUE));
                 wprintw(win, "%s ", line->timestr);
                 wattroff(win, COLOR_PAIR(BLUE));
@@ -669,14 +768,15 @@ void line_info_print(ToxWindow *self)
                 wprintw(win, "%s ", line->name1);
                 wattroff(win, A_BOLD);
 
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
                 waddch(win, '\n');
 
                 wattroff(win, COLOR_PAIR(line->colour));
 
                 break;
+            }
 
-            case NAME_CHANGE:
+            case NAME_CHANGE: {
                 wattron(win, COLOR_PAIR(BLUE));
                 wprintw(win, "%s ", line->timestr);
                 wattroff(win, COLOR_PAIR(BLUE));
@@ -687,7 +787,7 @@ void line_info_print(ToxWindow *self)
                 wprintw(win, "%s", line->name1);
                 wattroff(win, A_BOLD);
 
-                print_ret = print_wrap(win, line, max_x, max_y);
+                print_wrap(win, line, max_x, max_y);
 
                 wattron(win, A_BOLD);
                 wprintw(win, "%s\n", line->name2);
@@ -695,6 +795,7 @@ void line_info_print(ToxWindow *self)
                 wattroff(win, COLOR_PAIR(MAGENTA));
 
                 break;
+            }
         }
 
         line = line->next;
@@ -749,10 +850,9 @@ void line_info_set(ToxWindow *self, uint32_t id, char *msg)
 
     while (line) {
         if (line->id == id) {
-            size_t new_len = strlen(msg);
-            line->len = line->len - line->msg_len + new_len;
-            line->msg_len = new_len;
-            snprintf(line->msg, sizeof(line->msg), "%s", msg);
+            const uint16_t new_width = line_info_add_msg(line->msg, sizeof(line->msg) / sizeof(wchar_t), msg);
+            line->len = line->len - line->msg_width + new_width;
+            line->msg_width = new_width;
             return;
         }
 
