@@ -37,6 +37,7 @@
 #include <string.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <termios.h>
 #include <time.h>
@@ -53,6 +54,7 @@
 #include "execute.h"
 #include "file_transfers.h"
 #include "friendlist.h"
+#include "groupchats.h"
 #include "line_info.h"
 #include "log.h"
 #include "message_queue.h"
@@ -229,20 +231,55 @@ void exit_toxic_err(const char *errmsg, int errcode)
     exit(EXIT_FAILURE);
 }
 
+static const char *tox_log_level_show(Tox_Log_Level level)
+{
+    switch (level) {
+        case TOX_LOG_LEVEL_TRACE:
+            return "TRACE";
+
+        case TOX_LOG_LEVEL_DEBUG:
+            return "DEBUG";
+
+        case TOX_LOG_LEVEL_INFO:
+            return "INFO";
+
+        case TOX_LOG_LEVEL_WARNING:
+            return "WARNING";
+
+        case TOX_LOG_LEVEL_ERROR:
+            return "ERROR";
+    }
+
+    return "<invalid>";
+}
+
 void cb_toxcore_logger(Tox *m, TOX_LOG_LEVEL level, const char *file, uint32_t line, const char *func,
                        const char *message, void *user_data)
 {
-    UNUSED_VAR(user_data);
-    UNUSED_VAR(file);
     UNUSED_VAR(m);
 
-    if (user_data) {
-        FILE *fp = (FILE *)user_data;
-        fprintf(fp, "[%d] %u:%s() - %s\n", level, line, func, message);
-        fflush(fp);
-    } else {
-        fprintf(stderr, "[%d] %u:%s() - %s\n", level, line, func, message);
+    FILE *fp = (FILE *)user_data;
+
+    if (!fp) {
+        fp = stderr;
     }
+
+    struct timeval tv;
+
+    gettimeofday(&tv, NULL);
+
+    struct tm tmp;
+
+    gmtime_r(&tv.tv_sec, &tmp);
+
+    char timestamp[200];
+
+    strftime(timestamp, sizeof(timestamp), "%F %T", &tmp);
+
+    fprintf(fp, "%c %s.%06ld %s:%u(%s) - %s\n", tox_log_level_show(level)[0], timestamp, tv.tv_usec, file, line, func,
+            message);
+
+    fflush(fp);
 }
 
 /* Sets ncurses refresh rate. Lower values make it refresh more often. */
@@ -472,6 +509,17 @@ static void load_friendlist(Tox *m)
     sort_friendlist_index();
 }
 
+static void load_groups(Tox *m)
+{
+    size_t numgroups = tox_group_get_number_groups(m);
+
+    for (size_t i = 0; i < numgroups; ++i) {
+        if (init_groupchat_win(m, i, NULL, 0, Group_Join_Type_Load) != 0) {
+            tox_group_leave(m, i, NULL, 0, NULL);
+        }
+    }
+}
+
 static void load_conferences(Tox *m)
 {
     size_t num_chats = tox_conference_get_chatlist_size(m);
@@ -815,6 +863,22 @@ static void init_tox_callbacks(Tox *m)
     tox_callback_file_recv_control(m, on_file_recv_control);
     tox_callback_file_recv_chunk(m, on_file_recv_chunk);
     tox_callback_friend_lossless_packet(m, on_lossless_custom_packet);
+    tox_callback_group_invite(m, on_group_invite);
+    tox_callback_group_message(m, on_group_message);
+    tox_callback_group_private_message(m, on_group_private_message);
+    tox_callback_group_peer_status(m, on_group_status_change);
+    tox_callback_group_peer_join(m, on_group_peer_join);
+    tox_callback_group_peer_exit(m, on_group_peer_exit);
+    tox_callback_group_peer_name(m, on_group_nick_change);
+    tox_callback_group_topic(m, on_group_topic_change);
+    tox_callback_group_peer_limit(m, on_group_peer_limit);
+    tox_callback_group_privacy_state(m, on_group_privacy_state);
+    tox_callback_group_topic_lock(m, on_group_topic_lock);
+    tox_callback_group_password(m, on_group_password);
+    tox_callback_group_self_join(m, on_group_self_join);
+    tox_callback_group_join_fail(m, on_group_rejected);
+    tox_callback_group_moderation(m, on_group_moderation);
+    tox_callback_group_voice_state(m, on_group_voice_state);
 }
 
 static void init_tox_options(struct Tox_Options *tox_opts)
@@ -1563,7 +1627,7 @@ int main(int argc, char **argv)
     } else if (nameserver_ret == -2) {
         queue_init_message("Name lookup server list could not be found.");
     } else if (nameserver_ret == -3) {
-        queue_init_message("Name lookup server list does not contain any valid entries.");
+        fprintf(stderr, "Name lookup server list does not contain any valid entries\n");
     }
 
 #ifdef X11
@@ -1584,6 +1648,7 @@ int main(int argc, char **argv)
 
     prompt = init_windows(m);
     prompt_init_statusbar(prompt, m, !datafile_exists);
+    load_groups(m);
     load_conferences(m);
     set_active_window_index(0);
 
