@@ -23,6 +23,10 @@
 #ifndef TOXIC_H
 #define TOXIC_H
 
+#ifndef NCURSES_WIDECHAR
+#define NCURSES_WIDECHAR 1
+#endif
+
 #ifndef TOXICVER
 #define TOXICVER "NOVERS"    /* Use the -D flag to set this */
 #endif
@@ -40,13 +44,17 @@
 
 #include <tox/tox.h>
 
+#ifdef TOX_EXPERIMENTAL
+#include <tox/tox_private.h>
+#endif
+
 #define UNKNOWN_NAME "Anonymous"
 #define DEFAULT_TOX_NAME "Tox User"   /* should always be the same as toxcore's default name */
 
 #define MAX_STR_SIZE TOX_MAX_MESSAGE_LENGTH    /* must be >= TOX_MAX_MESSAGE_LENGTH */
 #define MAX_CMDNAME_SIZE 64
 #define TOXIC_MAX_NAME_LENGTH 32   /* Must be <= TOX_MAX_NAME_LENGTH */
-#define KEY_IDENT_DIGITS 3    /* number of hex digits to display for the pub-key based identifier */
+#define KEY_IDENT_BYTES 6    /* number of hex digits to display for the public key identifier */
 #define TIME_STR_SIZE 32
 #define COLOR_STR_SIZE 10 /* should fit every color option */
 
@@ -110,8 +118,8 @@ void flag_interface_refresh(void);
 /* Sets ncurses refresh rate. Lower values make it refresh more often. */
 void set_window_refresh_rate(size_t refresh_rate);
 
-void exit_toxic_success(Tox *m);
-void exit_toxic_err(const char *errmsg, int errcode);
+void exit_toxic_success(Tox *m) __attribute__((__noreturn__));
+void exit_toxic_err(const char *errmsg, int errcode) __attribute__((__noreturn__));
 
 int store_data(Tox *m, const char *path);
 
@@ -144,6 +152,32 @@ void on_file_recv(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint32_t k
 void on_friend_typing(Tox *m, uint32_t friendnumber, bool is_typing, void *userdata);
 void on_friend_read_receipt(Tox *m, uint32_t friendnumber, uint32_t receipt, void *userdata);
 void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *data, size_t length, void *userdata);
+void on_group_invite(Tox *m, uint32_t friendnumber, const uint8_t *invite_data, size_t length,
+                     const uint8_t *group_name,
+                     size_t group_name_length, void *userdata);
+void on_group_message(Tox *m, uint32_t groupnumber, uint32_t peernumber, TOX_MESSAGE_TYPE type,
+                      const uint8_t *message, size_t length, uint32_t message_id, void *userdata);
+void on_group_private_message(Tox *m, uint32_t groupnumber, uint32_t peernumber, TOX_MESSAGE_TYPE type,
+                              const uint8_t *message, size_t length,
+                              void *userdata);
+void on_group_peer_join(Tox *m, uint32_t groupnumber, uint32_t peernumber, void *userdata);
+void on_group_peer_exit(Tox *m, uint32_t groupnumber, uint32_t peer_id, Tox_Group_Exit_Type exit_type,
+                        const uint8_t *nick,
+                        size_t nick_len, const uint8_t *partmsg, size_t length, void *userdata);
+void on_group_topic_change(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *topic, size_t length,
+                           void *userdata);
+void on_group_peer_limit(Tox *m, uint32_t groupnumber, uint32_t peer_limit, void *userdata);
+void on_group_privacy_state(Tox *m, uint32_t groupnumber, Tox_Group_Privacy_State privacy_state, void *userdata);
+void on_group_topic_lock(Tox *m, uint32_t groupnumber, Tox_Group_Topic_Lock topic_lock, void *userdata);
+void on_group_password(Tox *m, uint32_t groupnumber, const uint8_t *password, size_t length, void *userdata);
+void on_group_nick_change(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *newname, size_t length,
+                          void *userdata);
+void on_group_status_change(Tox *m, uint32_t groupnumber, uint32_t peernumber, TOX_USER_STATUS status, void *userdata);
+void on_group_self_join(Tox *m, uint32_t groupnumber, void *userdata);
+void on_group_rejected(Tox *m, uint32_t groupnumber, Tox_Group_Join_Fail type, void *userdata);
+void on_group_moderation(Tox *m, uint32_t groupnumber, uint32_t source_peernum, uint32_t target_peernum,
+                         Tox_Group_Mod_Event type, void *userdata);
+void on_group_voice_state(Tox *m, uint32_t groupnumber, Tox_Group_Voice_State voice_state, void *userdata);
 
 extern char *DATA_FILE;
 extern char *BLOCK_FILE;
